/************************************************************************/
/*  Routine to read qmerge ascii header.				*/
/*									*/
/*	Douglas Neuhauser						*/
/*	Seismological Laboratory					*/
/*	University of California, Berkeley				*/
/*	doug@seismo.berkeley.edu					*/
/*									*/
/************************************************************************/

/*
 * Copyright (c) 1996-2000 The Regents of the University of California.
 * All Rights Reserved.
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes,
 * without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice, this paragraph and the
 * following three paragraphs appear in all copies.
 * 
 * Permission to incorporate this software into commercial products may
 * be obtained from the Office of Technology Licensing, 2150 Shattuck
 * Avenue, Suite 510, Berkeley, CA  94704.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
 * INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
 * ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
 * CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#ifndef lint
static char sccsid[] = "$Id: read_hdr.c,v 1.2 2000/10/26 12:21:04 doug Exp $ ";
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "qlib2.h"

#include "procs.h"
#include "externals.h"

int herrno;

#define	LINELEN 256

/************************************************************************/
/*  read_hdr:								*/
/*	Read header on data stream, fill in, and return ptr to DATA_HDR.*/
/************************************************************************/
DATA_HDR *read_hdr 
   (FILE	*input)		/* input FILE.				*/
{
    char line[LINELEN];
    char station[8] = "";
    char channel[8] = "";
    char network[4] = "";
    char location[4] = "";
    char stream[80];
    char rate_str[16];
    char date_str[40], time_str[40], *p;
    int rate, rate_mult;
    double drate;
    int aflag, ioflag, qflag, timecorr;
    int n = 0;
    DATA_HDR *hdr;
    INT_TIME it, *pt;

    /* Skip over any data until we find a line that looks like a valid	*/
    /* header line.  Valid header line looks like:			*/
    /* seq=1, station=ARC, channel=BHZ, time=1999.040 14:32:00.0029 to 1999.040 14:32:00.8029, corr=0, nsamp=16, aflag=00, ioflag=20, qflag=00, rate=20
    /* or
    /* seq=1 stream=BKS.BK.BHZ. time=1999.113 00:00:02.0406 to 1999.113 00:02:52.7406 corr=0 nsamp=3414 aflag=00 ioflag=20 qflag=00 rate=20
    */
    while (1) {
	n++;
	if (fgets (line, LINELEN, input) == NULL) return(NULL);
	if (line[0] == 's' && line[1] == 'e') break;
    }
#ifdef	DEBUG
    printf ("found header on line %d\n", n);
#endif

    if ((hdr = new_data_hdr()) == NULL) return (NULL);

    /* Parse the header line. */
    if (strstr(line, "station")) {
	n = sscanf (line, "%*s station=%s channel=%s time=%s %s to %*s %*s, corr=%d, nsamp=%*d, aflag=%x, ioflag=%x, qflag=%x, rate=%s",
		    station, channel, date_str, time_str,
		    &timecorr, &aflag, &ioflag, &qflag, rate_str);
	if (n != 9) {
	    fprintf (stderr, "Error parsing hdr line, expected 9, found %d\n", n);
	    exit(1);
	}
    }
    else if (strstr(line, "stream")) {
	char *p, *q;
	memset (stream, 0, sizeof(stream));
	n = sscanf (line, "%*s stream=%s time=%s %s to %*s %*s corr=%d nsamp=%*d aflag=%x ioflag=%x qflag=%x rate=%s",
		    stream, date_str, time_str,
		    &timecorr, &aflag, &ioflag, &qflag, rate_str);
	if (n != 8) {
	    fprintf (stderr, "Error parsing hdr line, expected 8, found %d\n", n);
	    exit(1);
	}
	p = stream;
	/* Get station */
	q = strchr(p,'.');
	if (q) *q = '\0';
	strcpy (station,p);
	p = (q) ? ++q : p + strlen(p);
	/* Get network */
	q = strchr(p,'.');
	if (q) *q = '\0';
	if (strlen(p)) strcpy (network,p);
	p = (q) ? ++q : p + strlen(p);
	/* Get channel */
	q = strchr(p,'.');
	if (q) *q = '\0';
	strcpy (channel,p);
	p = (q) ? ++q : p + strlen(p);
	/* Get location */
	q = strchr(p,'.');
	if (q) *q = '\0';
	if (strlen(p)) strcpy (location,p);
	p = (q) ? ++q : p + strlen(p);
    }
    
    strcat (date_str, ",");
    strcat (date_str, time_str);
    if ((pt = parse_date(date_str)) != NULL) {
	it = *pt;
    }
    else {
	fprintf (stderr, "Error parsing date string: %s\n", date_str);
	exit(1);
    }
    hdr->activity_flags = (unsigned int)aflag;
    hdr->io_flags = (unsigned int)ioflag;
    hdr->data_quality_flags = (unsigned int)qflag;
    if ((p=strchr(rate_str,'.')) != NULL) {
	double drate;
	int rate, rate_mult, n;
	drate = atof (rate_str);
	p++;
	n = strlen(p);
	rate_mult = pow (10, n);
	rate = drate * rate_mult;
	hdr->sample_rate = rate;
	hdr->sample_rate_mult = -1 * rate_mult;
    }
    else {
	int rate;
	rate = atoi (rate_str);
	hdr->sample_rate = rate;
	hdr->sample_rate_mult = 1;
    }
    strcpy (hdr->station_id, station);
    strcpy (hdr->channel_id, channel);
    if (strlen(network) > 0) strcpy (hdr->network_id, network);
    if (strlen(location) > 0) strcpy (hdr->location_id, location);
    hdr->begtime = it;
    /* Compute hdrtime from begtime and timecorr */
    if (! (hdr->activity_flags & ACTIVITY_TIME_CORR_APPLIED)) {
	it = add_dtime (it, -1 * timecorr * USECS_PER_TICK);
	hdr->hdrtime = it;
    }
    else {
	hdr->hdrtime = hdr->begtime;
    }
    hdr->num_blockettes = 0;
    hdr->pblockettes = NULL;
    hdr->blksize = blksize;
    hdr->seq_no = 1;

#ifdef	DEBUG
    fprintf ("%s.%s.%s.%s %s rate=%d,%d",
	     hdr->station_id, hdr->network_id, hdr->channel_id, hdr->location_id,
	     time_to_str(hdr->begtime), hdr->sample_rate, hdr->sample_rate_mult);
#endif
    return(hdr);
}
